// -*- coding: utf-8 -*-
// Time-stamp: <bicontainer.cpp   9 avr 2014 19:45:21> 

#include "bicontainer.hpp"

// Constructor
Bicontainer::Bicontainer(sf::Sprite* sprite0, sf::Sprite* sprite1, 
			 int xmax, int ymax, int n) 
  : Container(xmax, ymax) {
  // Fill the left half of the container with new particules
  // These particules are drown with the first sprite sprite0
  for (int i = 0; i < n; i++)
    add(Particule(sprite0, getxdim()/2, getydim()));
  // Fill the right half of the container with new particules
  // These particules are drown with the first sprite sprite1
  for (int i = 0; i < n; i++)
    add(Particule(sprite1, getxdim()/2, 0, getxdim(), getydim()));
}

// Move one particule in the container
void Bicontainer::move(Particule* particule, double speed) const {
  // Half size of the hole in the middle in percent
  double threshold = 0.05; 
  // New positions 
  double tx = particule->x + particule->dx*speed;
  double ty = particule->y + particule->dy*speed;
  // Check if left border is reached
  if (tx < 0) {
    tx = -tx;			// Symmetry with respect to 0
    // Reverse X speed
    particule->dx = -particule->dx;
  }
  // Check if middle border is reached from left
  if (particule->x < getxdim()/2 && tx > getxdim()/2) {
    // Y coordinate of the crossing with the verical line x = 1/2
    double yc = (particule->y * (tx - getxdim()/2) + 
		 ty * (getxdim()/2 - particule->x)) / (tx - particule->x);
    // Bounce if not in the hole
    if (yc > (getydim() * (0.5 + threshold)) || 
	yc < (getydim() * (0.5 - threshold))) {
      tx = getxdim() - tx;	// Symmetry with respect to xdim/2
      // Reverse X speed
      particule->dx = -particule->dx;
    }
  }
  // Check if middle border is reached from right
  if (particule->x > getxdim()/2 && tx < getxdim()/2) {
    // Y coordinate of the crossing with the verical line x = 1/2
    double yc = (particule->y * (tx - getxdim()/2) + 
		 ty * (getxdim()/2 - particule->x)) / (tx - particule->x);
    // Bounce if not in the hole
    if (yc > (getydim() * (0.5 + threshold)) || 
	yc < (getydim() * (0.5 - threshold))) {
      tx = getxdim() - tx;	// Symmetry with respect to xdim/2
      // Reverse X speed
      particule->dx = -particule->dx;
    }
  }
  // Check if right border is reached
  if (tx > getxdim()) {
    tx = 2*getxdim() - tx;	// Symmetry with respect to xdim
    // Reverse X speed
    particule->dx = -particule->dx;
  }
  // Check if bottom border is reached
  if (ty < 0) {
    ty = -ty;			// Symmetry with respect to 0
    // Reverse Y speed
    particule->dy = -particule->dy;
  }
  // Check if top border is reached
  if (ty > getydim()) {
    ty = 2*getydim() - ty;	// Symmetry with respect to ydim
    // Reverse Y speed
    particule->dy = -particule->dy;
  }
  // Update new position
  particule->x = tx;
  particule->y = ty;
}
