// -*- coding: utf-8 -*-
// Time-stamp: <container.cpp  27 mar 2014 07:12:20> 

#include "container.hpp"

// Constructor
Container::Container(sf::Sprite* sprite, int xmax, int ymax, int n) 
  : xdim(xmax), ydim(ymax) {
  // Fill the vector with new particules
  for (int i = 0; i < n; i++)
    add(Particule(sprite, xdim, ydim));
}

// Destructor
Container::~Container() {
  // Nothing to be done
}

// Move all particules in the container
void Container::move(double speed) {
  for (Iterator it = begin(); it != end(); ++it)
    // Move each particule
    move(&*it, speed);
}

// Move one particule in the container
void Container::move(Particule* particule, double speed) const {
  // Move the particule
  particule->x = particule->x + particule->dx*speed;
  particule->y = particule->y + particule->dy*speed;
  // Test X boundary
  if (particule->x < 0 || particule->x > xdim) {
    // Reverse X speed
    particule->dx = -particule->dx;
    if (particule->x < 0)
      particule->x = -particule->x;         // Symmetry with respect to 0
    else
      particule->x = 2*xdim - particule->x; // Symmetry with respect to xdim
  }
  // Test Y boundary
  if (particule->y < 0 || particule->y > ydim) {
    // Reverse Y speed
    particule->dy = -particule->dy;
    if (particule->y < 0)
      particule->y = -particule->y;         // Symmetry with respect to 0
    else
      particule->y = 2*ydim - particule->y; // Symmetry with respect to ydim
  }
}
